<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Tco01300ListaPrecosController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService; 
    protected $parameterBag; 
    protected $environment; 
    protected $entityManager; 
    
    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService   = $crudService; 
        $this->parameterBag  = $parameterBag; 
        $this->environment   = $environment; 
        $this->entityManager = $entityManager; 
    }
    
    public function atualizaListaPrecos(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();
        $regioesAtualizadas = array();
        
        if($row["TODAS_REGIOES"] == "T"){
            $row["CDREGIAO"] = $this->retornaRegioesDisponiveis($connection);
        }
        
        try{
            
            foreach($row["CDREGIAO"] as $regiao) {
                try{
                    $this->entityManager->beginTransaction();
                    $rowParaAtualizar = array(
                        "DTREFERENCIA" => $row["DTREFERENCIA"],
                        "CDREGIAO" => $regiao["CDREGIAO"]
                    );
                    
                    $result = $this->procedureAtualizaListaPrecos($connection, $rowParaAtualizar);
                    $this->entityManager->commit();
                    
                    if($result["P_NRLISTAPRECOTCO"]){
                        array_push($regioesAtualizadas, array(
                            "CDREGIAO" =>          $regiao["CDREGIAO"],
                            "NMREGIAO" =>          $regiao["NMREGIAO"],
                            "P_NRLISTAPRECOTCO" => $result["P_NRLISTAPRECOTCO"]
                        ));
                    }
                } catch(\Exception $e) {
                    $this->entityManager->rollback();
                    $response->addDataSet(new DataSet('regioes_atualizadas', $regioesAtualizadas));
                    $response->setError(new Error($e->getMessage(), 500));
                }
            }
            
            $response->addDataSet(new DataSet('regioes_atualizadas', $regioesAtualizadas)); 
        } catch(\Exception $e) {
            $response->addDataSet(new DataSet('regioes_atualizadas', $regioesAtualizadas));
            $response->setError(new Error($e->getMessage(), 500));
        }
        
    }
    
    private function procedureAtualizaListaPrecos($connection, $row) {
        $procedure = new StoredProcedure($connection, 'GRAVA_LISTAPRECOTCO');
        
        $procedure->addParam(new Param('P_DTREFERENCIA',     Param::PARAM_INPUT,  $row['DTREFERENCIA'],                Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDREGIAO',         Param::PARAM_INPUT,  $row['CDREGIAO'],                    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG',            Param::PARAM_INPUT,  $this->environment->getNrOrgTrab(),  Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_CDOPERADOR',       Param::PARAM_INPUT,  $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLISTAPRECOTCO',  Param::PARAM_OUTPUT, null,                                Param::PARAM_TYPE_INT, 20));
        
        $result = $procedure->execute();
        
        return $result;
    }
    
    private function retornaRegioesDisponiveis($connection) {
        $sql = "SELECT CDREGIAO, 
                       NMREGIAO,
                       NRORG
                  FROM REGIAO
                 WHERE NRORG = :NRORG";
                  
        $params = array(
            "NRORG" => $this->environment->getNrOrgTrab()
        );
                   
        $query = $connection->prepare($sql);
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    /*GRAVA_LISTAPRECOTCO
    (
    P_DTREFERENCIA IN DATE,
    P_CDREGIAO IN VARCHAR2,
    P_NRORG IN NUMBER,
    P_CDOPERADOR IN VARCHAR2,
    P_NRLISTAPRECOTCO OUT NUMBER
    )*/

}