<?php

namespace Teknisa\TCO\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Tco01101DetalhesServicoService extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $tcoUtil;
    protected $tcoList;
    protected $tcoMessage;
    protected $connection;
    protected $environment;
    protected $entityManager;
    protected $dataSourceName;
    protected $dataSourceManager;
    protected $tco01100GeneralLibPropCom;

    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $newCode, $tcoUtil, $tcoList, $tcoMessage, $tco01100GeneralLibPropCom) {
        parent::__construct($crudService, $parameterBag);

        $this->tcoUtil                          = $tcoUtil;
        $this->newCode                          = $newCode;
        $this->tcoList                          = $tcoList;
        $this->tcoMessage                       = $tcoMessage;
        $this->connection                       = $entityManager->getConnection();
        $this->crudService                      = $crudService;
        $this->environment                      = $environment;
        $this->entityManager                    = $entityManager;
        $this->dataSourceManager                = $dataSourceManager;
        $this->tco01100GeneralLibPropCom        = $tco01100GeneralLibPropCom;
    }

    /**
     * Salva alteração no registro de serviço
     * da proposta.
     * 
     * @param Request $request Requisição do cliente.
     */
    public function saveEditServicoProposta($request){
        $row = $request->getRow();
        $row->set('__is_new', false);
        
        if(!$row->has('QTTURNOS') || !$row->get('QTTURNOS'))
            $row->set('QTTURNOS', 1);
        
        $reqRPath   = $request->getRoutePath();
        $reqMethod  = $request->getMethod();
        $reqUserId  = $request->getUserId();

        $request   = new DTO\Request\Row($row, $reqMethod, $reqRPath, $reqUserId);
        $rowToSave = $this->crudService->getRowsToSave($request, 'servicopropcom');
        $dataSet   = new DataSet('servicopropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);

        $this->clearCardTrabFromServico($row->get('NRDETALHESERVPROPCOM'));
        $this->setSituacaoServPropCom($row->get('NRSERVICOPROPCOM'), 'PENDENTE');
    }

    /**
     * Salva alteração nos registros de 
     * tipo de prato do serviço da proposta.
     * 
     * @param array[DTO\Row] $rows
     * @param array $reqParams Parametros da
     * requisição do usuário.
     */
    public function saveEditTipoPrat($rows, $reqParams){
        foreach ($rows as $index => $row) {
            if($row->get('changed') && !$this->isServCompl($row->get('NRSERVICOPROPCOM'))){
                if($row->get('__isSelected')){
                    $this->insertConfigServPropCom($row, $reqParams);
                } else {
                    $this->removeConfigServPropCom($row, $reqParams);
                }
            }

            $this->removeConfGrupoTPPropCom($row, $reqParams);
            $this->clearCardTrabFromServico($row->get('NRDETALHESERVPROPCOM'));
            $this->setSituacaoServPropCom($row->get('NRSERVICOPROPCOM'), 'PENDENTE');
        }
    }

    /**
     * Verifica se serviço é complementar.
     *
     * @param int|float $nrServicoPropCom
     * 
     * @return bool retorna se serviço é
     * complementar.
     */
    private function isServCompl($nrServicoPropCom){
        $servPropCom = $this->tco01100GeneralLibPropCom->getServPropComComplete($nrServicoPropCom);
        return (sizeof($servPropCom) && $servPropCom[0]['NRSERVICOPROPCOMCOMPL']);
    }

    /**
     * Insere novo registro de configuração
     * do serviço da proposta comercial.
     * 
     * @param DTO\Row $row
     * @param array $reqParams Parametros da
     * requisição.
     */
    public function insertConfigServPropCom($row, $reqParams){
        $nrOrg                = $this->environment->getNrOrg();
        $nrConfigServPropCom  = $this->newCode->getCodeByTableName('CONFIGSERVPROPCOM', $nrOrg, 1, 10, true);
        $row->set('NRCONFIGSERVPROPCOM', (float)$nrConfigServPropCom);
        $row->set('__is_new', true);

        $request   = new DTO\Request\Row($row, $reqParams["REQMETHOD"], $reqParams["REQRPATH"], $reqParams["REQUSERID"]);
        $rowToSave = $this->crudService->getRowsToSave($request, 'configservpropcom');
        $dataSet   = new DataSet('configservpropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
        
    }

    /**
     * Remove registro de configuração
     * do serviço da proposta comercial.
     * 
     * @param DTO\Row $row
     * @param array $reqParams Parametros da
     * requisição.
     * 
     */
    public function removeConfigServPropCom($row, $reqParams){
        if($row->has('NRCONFIGSERVPROPCOM') && $row->get('NRCONFIGSERVPROPCOM')){
            $dataSet   = new DataSet('configservpropcom', array($row));
            $this->dataSourceManager->delete($dataSet);
        }
    }

    /**
     * Salva alterações de porcionamento e gramatura
     * para o grupo de tipo de prato relacionado.
     * 
     * @param DTO\Row $row
     * @param array $reqParams Parametros da
     * requisição do usuário.
     */
    public function saveAddPorcGram($row, $reqParams){
        if(!$row->has('NRCONFGRUPOTPPROPCOM') || !$row->get('NRCONFGRUPOTPPROPCOM')){
            $nrOrg                = $this->environment->getNrOrg();
            $nrConfGrupoTPPropCom  = $this->newCode->getCodeByTableName('CONFGRUPOTPPROPCOM', $nrOrg, 1, 10, true);
            $row->set('NRCONFGRUPOTPPROPCOM', (float)$nrConfGrupoTPPropCom);
            $row->set('__is_new', true);
        }

        $request   = new DTO\Request\Row($row, $reqParams["REQMETHOD"], $reqParams["REQRPATH"], $reqParams["REQUSERID"]);
        $rowToSave = $this->crudService->getRowsToSave($request, 'confgrupotppropcom');
        $dataSet   = new DataSet('confgrupotppropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);

        $this->clearCardTrabFromServico($row->get('NRDETALHESERVPROPCOM'));
        $this->setSituacaoServPropCom($row->get('NRSERVICOPROPCOM'), 'PENDENTE');
    }

    /**
     * Remove registro de configuração
     * de grupo de tipo de prato.
     * 
     * @param DTO\Row $row
     * @param array $reqParams Parametros da
     * requisição.
     * 
     */
    private function removeConfGrupoTPPropCom($row, $reqParams){
        $cdGrTipoPrat = $this->tco01100GeneralLibPropCom->getData('tipoprat', array('CDTIPOPRAT'=>$row->get('CDTIPOPRAT')));
        if($cdGrTipoPrat){
            $cdGrTipoPrat = $cdGrTipoPrat[0]['CDGRTIPOPR'];
        }
        
        $grTipoPrat = $this->tco01100GeneralLibPropCom->getData('confgrupotppropcom', array(
            'CDGRTIPOPR'=>$cdGrTipoPrat,
            'NRDETALHESERVPROPCOM'=>$row->get('NRDETALHESERVPROPCOM')
        ));
        
        if($grTipoPrat){
            $row->set('CDGRTIPOPR', $grTipoPrat[0]['CDGRTIPOPR']);
            $row->set('NRCONFGRUPOTPPROPCOM', $grTipoPrat[0]['NRCONFGRUPOTPPROPCOM']);
            $row->set('NRPORCIONAMENTO', $grTipoPrat[0]['NRPORCIONAMENTO']);
            $row->set('__is_new', false);
            
            $dataSet   = new DataSet('confgrupotppropcom', array($row));
            $this->dataSourceManager->delete($dataSet);
        }

        $this->clearCardTrabFromServico($row->get('NRDETALHESERVPROPCOM'));
        $this->setSituacaoServPropCom($row->get('NRSERVICOPROPCOM'), 'PENDENTE');
    }

    /**
     * Executa a rotina de calculo do custo do
     * serviço.
     * 
     * @param DTO\Request $tipos de prato marcadas no
     * serviço da proposta.
     */
    public function calcularCustServ($row){
        $propostaComercial = $this->getPropostaComercialByNrVersao($row->get('NRVERSAOPROPCOM'));
        $nrAtivCliSegMer = $propostaComercial ? $this->getAtivCliSegMerByNrVerPropCom($propostaComercial[0]) : null;
        $regiao = $propostaComercial ? $this->getRegiaoByMunicipio($propostaComercial[0]['CDMUNICIPIO']) : null;
        $configServPropCom = $this->getConfigServPropCom($row->get('NRDETALHESERVPROPCOM')) ?: array();
        $today = $this->environment->getCurrentTime()->format('d/m/Y');
        $todosGrupos = $this->getGrupos($row->get('NRDETALHESERVPROPCOM'));
        if($nrAtivCliSegMer && sizeof($nrAtivCliSegMer)){
            $nrAtivCliSegMer = (int)$nrAtivCliSegMer[0]['NRATIVCLISEGMER'];
        }

        $configServPropCom = is_array($configServPropCom) ? array_map(function($config){
            return $config['NRCONFIGCARDBASE'];
        }, $configServPropCom) : array();
        if(count($configServPropCom) === 0){
            return new DataSet("faltaInfo", ['TCO01101_SELECIONE_TIPOS_PRATO']);
        }
        $configServPropComList = $this->createLista($configServPropCom);
        
        $valueMapIdtipogramatura = array("INNATURA" => "0", "LIMPA" => "1", "PRONTA" => "2");
        
        $existe_vazio = false;
        $grupos = array();
        foreach ($todosGrupos as $gp){
            $idtipogramatura = "";
            if(!empty($gp['IDTIPOGRAMATURA'])){
                $idtipogramatura = $valueMapIdtipogramatura[$gp['IDTIPOGRAMATURA']];
            }
            
            if(empty($gp['NRPORCIONAMENTO'])){
                $existe_vazio = true;
            }
                
            $grupos[] = $gp['CDGRTIPOPR'].'|'.$gp['NRPORCIONAMENTO'].'|'.$gp['NRGRAMATURACARDBASE'].'|'.$idtipogramatura;
        };
        
        if((count($grupos) === 0) || $existe_vazio){
            return new DataSet("faltaInfo", ['TCO01101_INFORME_PORC_GRAM']);
        }
        $grupos = $this->createLista($grupos);

        try{
            $this->entityManager->beginTransaction();
            $this->clearCardTrabFromServico($row->get('NRDETALHESERVPROPCOM'));
            $nrCardTrabCompl = $row->has('NRSERVICOPROPCOMCOMPL') ? $this->getNrCardTrab($row->get('NRSERVICOPROPCOMCOMPL')) : null;

            $procedure = new StoredProcedure($this->connection, 'IMPORTA_CARDBASE_NO_CARDTRAB');
            
            $procedure->addParam(new Param('P_NRCARDAPIOBASE',          Param::PARAM_INPUT,  $row->get('NRCARDAPIOBASE'),         Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_IDORIGEM',                Param::PARAM_INPUT,  'CALCULOGN',                         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRATIVCLISEGMER',         Param::PARAM_INPUT,  $nrAtivCliSegMer,                    Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_CDREGIAO',                Param::PARAM_INPUT,  $regiao,                             Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPEACRESCUSTO',          Param::PARAM_INPUT,  '0',                                 Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRTIPOPUBLICO',           Param::PARAM_INPUT,  $row->get('NRTIPOPUBLICO'),          Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_QTSEGSEX',                Param::PARAM_INPUT,  $row->get('QTSEGSEX') ?: 0,          Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_QTSABADO',                Param::PARAM_INPUT,  $row->get('QTSABADO') ?: 0,          Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_QTDOMFERI',               Param::PARAM_INPUT,  $row->get('QTDOMFERI') ?: 0,         Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_QTTURNOS',                Param::PARAM_INPUT,  $row->get('QTTURNOS') ?: 0,          Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_CONFIGURACOES',           Param::PARAM_INPUT,  $configServPropComList,              Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_GRUPOS',                  Param::PARAM_INPUT,  $grupos,                             Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTCUSTO',                 Param::PARAM_INPUT,  $today,                              Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCARDTRABCOMPLEMENTADO', Param::PARAM_INPUT,  $nrCardTrabCompl,                    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRORG',                   Param::PARAM_INPUT,  $this->environment->getNrOrgTrab(),  Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_CDOPERADOR',              Param::PARAM_INPUT,  $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIALLOGADA',          Param::PARAM_INPUT,  $this->environment->getCdFilial(),   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCARDAPIOTRAB',          Param::PARAM_OUTPUT, null,                                Param::PARAM_TYPE_STR, 20));
            $procedure->addParam(new Param('P_LISTAERROS',              Param::PARAM_OUTPUT, null,                                Param::PARAM_TYPE_STR, 20));
            
            $result = $procedure->execute();
            
            $this->tcoList->deleteLista($configServPropComList);
            $this->tcoList->deleteLista($grupos);

            if(!empty($result['P_LISTAERROS'])){

                $struct = array(
                    'TIPO' => 'LISTA',
                    'CDFILIAL' => $this->environment->getCdFilial(),
                    'NRLISTA'  => $result['P_LISTAERROS']
                );  
                $this->entityManager->rollback();

                $this->updateCDFilialLista($row, $result['P_LISTAERROS']);

                return new DataSet("errorCode", $this->tcoMessage->convertMessage($struct));

            }else{

                $this->setCardTrabOnDetalheServPropCom($row, $result);
                $this->setSituacaoServPropCom($row->get('NRSERVICOPROPCOM'), 'CALCULADO');
                $this->entityManager->commit();
                $result['detalheServ'] = $this->tco01100GeneralLibPropCom->getServPropComComplete($row->get('NRSERVICOPROPCOM'))[0];
                return new DataSet("procedureResult", $result);

            }
            
        } catch (\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
            
    }
       
    /**
     * Atualiza lista de erro de calculo na tabela
     * de detalheservpropcom
     * 
     * @param ArrayObject $row
     * @param int|float $nrListaErro
     */
    private function updateCDFilialLista($row, $nrListaErro){
        try{
            $sql = "UPDATE DETALHESERVPROPCOM 
                SET CDFILIALLISTA = :CDFILIALLISTA,
                    NRLISTA = :NRLISTA,
                    NRORGULTATU = :NRORG,
                    CDOPERULTATU = :CDOPERADOR,
                    DTULTATU = SYSDATE
                WHERE NRDETALHESERVPROPCOM = :NRDETALHESERVPROPCOM";

            $params = array(
                "CDFILIALLISTA" => $this->environment->getCdFilial(),
                "NRLISTA" => $nrListaErro,
                "NRDETALHESERVPROPCOM" => $row['NRDETALHESERVPROPCOM'],
                "NRORG" => $this->environment->getNrOrgTrab(),
                "CDOPERADOR" => $this->environment->getCdoperador()
            );

            $query = $this->connection->prepare($sql);
            $query->execute($params);

        } catch (\Exception $e){
            throw $e;
        }
    }


    /**
     * Retorna numero do cardapio de trabalho do serviço
     * da proposta.
     */
    private function getNrCardTrab($nrServicoPropCom){
        if($nrServicoPropCom){

            $servPropCom = $this->tco01100GeneralLibPropCom->getServPropComComplete($nrServicoPropCom);
            return $servPropCom ? $servPropCom[0]['NRCARDAPIOTRAB'] : false;

        } else return false;
    }

    /**
     * Define o cardápio de trabalho no detalhe de servico
     * da proposta comercial, após execução da procedure
     * de calculo (IMPORTA_CARDBASE_NO_CARDTRAB).
     * 
     * @param object $procRow Row utilizada na execução
     * da procedure
     * @param object $procResult Resultado da procedure
     * quando realizada com sucesso.
     */
    private function setCardTrabOnDetalheServPropCom($procRow, $procResult){
        $nrDetalheServPropCom = $procRow->get('NRDETALHESERVPROPCOM');
        $nrCardTrab = $procResult['P_NRCARDAPIOTRAB'];
        
        $sql = 'UPDATE DETALHESERVPROPCOM
            SET NRCARDAPIOTRAB = :NRCARDAPIOTRAB,
                CDFILIALLISTA = NULL,
                NRLISTA = NULL,
                NRORGULTATU = :NRORG,
                CDOPERULTATU = :CDOPERADOR,
                DTULTATU = SYSDATE
            WHERE NRDETALHESERVPROPCOM = :NRDETALHESERVPROPCOM';

        $params = array(
            "NRCARDAPIOTRAB" => $nrCardTrab,
            "NRDETALHESERVPROPCOM" => $nrDetalheServPropCom,
            "NRORG" => $this->environment->getNrOrgTrab(),
            "CDOPERADOR" => $this->environment->getCdoperador()
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    /**
     * Define situação do serviço de acordo
     * com o escolhido.
     * 
     * @param int $nrServPropCom Numero do servico
     * @param string $sitServPropCom Situação do serviço.
     */
    public function setSituacaoServPropCom($nrServPropCom, $sitServPropCom){
        $sql = 'UPDATE SERVICOPROPCOM
            SET IDSERVICOPROPCOM = :IDSERVICOPROPCOM,
                NRORGULTATU = :NRORG,
                CDOPERULTATU = :CDOPERADOR,
                DTULTATU = SYSDATE
            WHERE NRSERVICOPROPCOM = :NRSERVICOPROPCOM';

        $params = array(
            "NRSERVICOPROPCOM" => $nrServPropCom,
            "IDSERVICOPROPCOM" => $sitServPropCom,
            "NRORG" => $this->environment->getNrOrgTrab(),
            "CDOPERADOR" => $this->environment->getCdoperador()
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    /**
     * Retorna grupos de tipo de prato, porcionamento
     * e gramatura relacionadas a ele.
     * 
     * @param int|float $nrDetalheServPropCom
     * 
     * @return bool|array Array de dados ou false
     * caso não haja registros.
     */
    public function getGrupos($nrDetalheServPropCom){
        
        $sql = "SELECT DISTINCT GRTP.CDGRTIPOPR,
                       GRTP.NMGRTIPOPR,
                       GRTP.NRORG,
                       GRTP.IDATIVO,
                       CGTPSPC.NRCONFGRUPOTPPROPCOM,
                       CGTPSPC.NRPORCIONAMENTO,
                       PC.CDPORCIONAMENTO,
                       PC.NMPORCIONAMENTO,
                       CGTPSPC.NRGRAMATURACARDBASE,
                       GMCB.VRGRAMATURA,
                       GMCB.SGUNIDADE,
                       CGTPSPC.IDTIPOGRAMATURA,
                       SPC.NRDETALHESERVPROPCOM,
                       SPC.NRSERVICOPROPCOM
                  FROM CONFIGSERVPROPCOM CSPC
                       INNER JOIN DETALHESERVPROPCOM DTSPC
                          ON CSPC.NRDETALHESERVPROPCOM = DTSPC.NRDETALHESERVPROPCOM
                       LEFT JOIN SERVICOPROPCOM SPC
                          ON SPC.NRSERVICOPROPCOM      = DTSPC.NRSERVICOPROPCOM AND 
                             SPC.NRDETALHESERVPROPCOM = DTSPC.NRDETALHESERVPROPCOM
                       LEFT JOIN TIPOPRAT TP
                          ON TP.CDTIPOPRAT = CSPC.CDTIPOPRAT
                       LEFT JOIN GRTIPOPR GRTP
                          ON GRTP.CDGRTIPOPR = TP.CDGRTIPOPR
                       LEFT JOIN CONFGRUPOTPPROPCOM CGTPSPC
                          ON ( CGTPSPC.NRDETALHESERVPROPCOM = DTSPC.NRDETALHESERVPROPCOM AND
                          CGTPSPC.CDGRTIPOPR            = GRTP.CDGRTIPOPR )
                       LEFT JOIN PORCIONAMENTO PC
                          ON PC.NRPORCIONAMENTO = CGTPSPC.NRPORCIONAMENTO
                       LEFT JOIN GRAMATURACARDBASE GMCB
                          ON GMCB.NRGRAMATURACARDBASE = CGTPSPC.NRGRAMATURACARDBASE
                       LEFT JOIN CONFIGCARDBASE
                          ON CONFIGCARDBASE.NRCONFIGCARDBASE = CSPC.NRCONFIGCARDBASE
                 WHERE DTSPC.NRDETALHESERVPROPCOM = :NRDETALHESERVPROPCOM
                   AND CONFIGCARDBASE.IDCLASSIFICACAO = 'ALIMENTAR'";

        $params = array(
            "NRDETALHESERVPROPCOM"=> $nrDetalheServPropCom
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    /**
     * Cria lista de valores.
     * 
     * @param array $dataArray
     */
    private function createLista($dataArray) {
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        
        foreach($dataArray as $data) {
            $this->tcoList->insertLista($nrlista, $data);
        }
        
        return $nrlista;
    }

    /**
     * Retorna registro da tabela AtivCliSegMer
     * de acordo com a proposta comercial.
     * 
     * @param int|float $propostaComercial
     * 
     * @return object|bool Registro de AtivClie caso
     * seja encontrado, false caso não exista.
     */
    private function getAtivCliSegMerByNrVerPropCom($propostaComercial){
        if($propostaComercial){
            return $this->tco01100GeneralLibPropCom->getData('ativclisegmer', array(
                'CDATIVCLI'=>$propostaComercial['CDATIVCLI'],
                'CDSEGMER'=>$propostaComercial['CDSEGMER']
            ));
        }

        return false;
    }

    /**
     * Retorna proposta comercial pelo numero
     * da versão.
     *
     * @param int|float $nrversao
     * 
     * @return object|bool Registro de propostacomercial
     * caso seja encontrado, false caso não exista.
     */
    private function getPropostaComercialByNrVersao($nrversaopropcom){
        $versaoPropCom = $this->tco01100GeneralLibPropCom->getData('versaopropcom', array(
            'NRVERSAOPROPCOM'=>$nrversaopropcom
        ));

        if($versaoPropCom && sizeof($versaoPropCom)){
            return $this->tco01100GeneralLibPropCom->getData('propostacomercial', array(
                'NRPROPOSTACOMERCIAL'=>$versaoPropCom[0]['NRPROPOSTACOMERCIAL']
            ));
        }

        return false;
    }

    /**
     * Retorna região de acordo com código do
     * municipio.
     * 
     * @param int|float $cdMunicipio
     */
    private function getRegiaoByMunicipio($cdMunicipio){
        $municipio =  $this->tco01100GeneralLibPropCom->getData('municipio', array(
            'CDMUNICIPIO'=>$cdMunicipio
        ));

        if($municipio && sizeof($municipio)){
            return $municipio[0]['CDREGIAO'];
        } else return false;
    }

    /**
     * Retorna lista de tipos de prato marcadas no
     * serviço da proposta.
     * 
     * @param int|float $nrDetalheServPropCom
     * 
     * @return array Registros de cardápio base.
     */
    private function getConfigServPropCom($nrDetalheServPropCom){
        return $this->tco01100GeneralLibPropCom->getData('configservpropcom', array(
            'NRDETALHESERVPROPCOM'=>$nrDetalheServPropCom
        ));
    }

    /**
     * Limpa registro de cardapio de trabalho do servico
     * passado por parametro.
     * 
     * @param int|float $nrDetalheServPropCom
     */
    public function clearCardTrabFromServico($nrDetalheServPropCom){
        $detalhe = $this->tco01100GeneralLibPropCom->getData('detalheservpropcom', array(
            'NRDETALHESERVPROPCOM'=>$nrDetalheServPropCom
        ));

        if($detalhe && sizeof($detalhe)){
            $nrCardTrab = $detalhe[0]['NRCARDAPIOTRAB'];
            
            $sql = "BEGIN
                UPDATE DETALHESERVPROPCOM SET NRCARDAPIOTRAB = NULL WHERE NRDETALHESERVPROPCOM = :NRDETALHESERVPROPCOM;
                DELETA_CARDTRAB(:NRCARDAPIOTRAB);
                DELETE FROM INCIDENCIAPROPCOM WHERE NRDETALHESERVPROPCOM = :NRDETALHESERVPROPCOM;
                DELETE FROM PRODUTOPROPCOM WHERE NRDETALHESERVPROPCOM = :NRDETALHESERVPROPCOM;
            END;";

            $params = array(
                "NRCARDAPIOTRAB" => $nrCardTrab,
                "NRDETALHESERVPROPCOM" => $nrDetalheServPropCom
            );

            $query = $this->connection->prepare($sql);
            $query->execute($params);
        }

    }

    /**
     * Salva alteração de incidencias para cada 
     * registro, excluindo caso solicitado for
     * 0, e inserindo caso for novo.
     * 
     * @param array $rows
     * @param array $reqParams
     */
    public function saveIncidencias($rows, $reqParams){
        foreach ($rows as $key => $row) {
            if($row->has('changed') && $row->get('changed')){
                if($row->get('QTSOLICITADA')!=null){
                    $this->upsertRegisterIncidencia($row, $reqParams); 
                } else {
                    $this->deleteRegisterIncidencia($row, $reqParams); 
                }
            }
        }
    }

    /**
     * Insere ou altera registro de incidencia.
     * 
     * @param object $row
     * @param array $reqParams
     */
    public function upsertRegisterIncidencia($row, $reqParams){
        
        if($row->has('NRINCIDENCIAPROPCOM') && $row->get('NRINCIDENCIAPROPCOM')){
            $row->set('__is_new', false);
        } else {
            $nrOrg                = $this->environment->getNrOrg();
            $nrincidenciapropcom  = $this->newCode->getCodeByTableName('INCIDENCIAPROPCOM', $nrOrg, 1, 10, true);
        
            $row->set('__is_new', true);
            $row->set('NRINCIDENCIAPROPCOM', $nrincidenciapropcom);
        }

        $request   = new DTO\Request\Row($row, $reqParams["REQMETHOD"], $reqParams["REQRPATH"], $reqParams["REQUSERID"]);
        $rowToSave = $this->crudService->getRowsToSave($request, 'incidenciapropcom');
        $dataSet   = new DataSet('incidenciapropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
    }


    /**
     * Remove registro de incidencia.
     * 
     * @param object $row
     * @param array $reqParams
     */
    public function deleteRegisterIncidencia($row, $reqParams){
        if($row->has('NRINCIDENCIAPROPCOM') && $row->get('NRINCIDENCIAPROPCOM')){
            $dataSet   = new DataSet('incidenciapropcom', array($row));
            $this->dataSourceManager->delete($dataSet);
        }
    }

    /**
     * Retorna dataset de marcas disponíveis para troca 
     * do produto.
     * 
     * @param array $conditions
     */
    public function getMarcaProduto($conditions){
        $sql = "SELECT DISTINCT P.CDMARCA, M.NMMARCA
                  FROM DETALHESERVPROPCOM D
                       INNER JOIN CARDAPIOTRAB C ON C.NRCARDAPIOTRAB = D.NRCARDAPIOTRAB
                       INNER JOIN PRODLISTAPRECOTCO P ON P.NRLISTAPRECOTCO = C.NRLISTAPRECOTCO 
                       INNER JOIN MARCA M ON M.CDMARCA = P.CDMARCA
                 WHERE D.NRDETALHESERVPROPCOM = :NRDETALHESERVPROPCOM
                   AND P.CDPRODESTO = :CDPRODUTO
                 ORDER BY M.NMMARCA";

        $params = array(
            "CDPRODUTO"=> $conditions[0]['value'],
            "NRDETALHESERVPROPCOM"=> $conditions[1]['value']
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    /**
     * Define marca para produto da proposta comercial.
     * 
     * @param object $row
     * @param array $reqParams Parametros da requisição.
     */
    public function selectMarca($row, $reqParams){
        if($row->has('NRPRODUTOPROPCOM') && $row->get('NRPRODUTOPROPCOM')){
            $row->set('__is_new', false);
        } else {
            $nrOrg = $this->environment->getNrOrg();
            $nrprodutopropcom  = $this->newCode->getCodeByTableName('produtopropcom', $nrOrg, 1, 10, true);
        
            $row->set('__is_new', true);
            $row->set('NRPRODUTOPROPCOM', $nrprodutopropcom);
        }
        $request   = new DTO\Request\Row($row, $reqParams["REQMETHOD"], $reqParams["REQRPATH"], $reqParams["REQUSERID"]);
        $rowToSave = $this->crudService->getRowsToSave($request, 'produtopropcom');
        $dataSet   = new DataSet('produtopropcom', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
    }


    /**
     * Limpa marca no produto da proposta comercial.
     * 
     * @param object $row
     * @param array $reqParams Parametros da requisição.
     */
    public function clearMarcaProduto($row){
        if($row->has('NRPRODUTOPROPCOM') && $row->get('NRPRODUTOPROPCOM')){
            $dataSet   = new DataSet('produtopropcom', array($row));
            $this->dataSourceManager->delete($dataSet);
        }
    }

    /**
     * Rotina de notificar planejamento, salvando
     * dados da tabela "HISTNOTIFPROPCOM".
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function notificarPlan(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $row->set('IDORIGEM', 'GN');
        $row->set('IDDESTINO', 'PLANEJAMENTO');
        $row->set('IDMOTIVO', $row->get('MOTIVO'));

        $request = new DTO\Request\Row($row, $request->getRoutePath(), $request->getMethod(), $request->getUserId());
        $procRet = $this->tco01100GeneralLibPropCom->notificarPlan($row, $request);

        if($procRet instanceof Error){
            $this->notificarPlanErrorHandler($procRet, $response);
        }
    }

    /**
     * Rotina de notificar planejamento, salvando
     * dados da tabela "HISTNOTIFPROPCOM".
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function confirmCustomize(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $row->set('IDORIGEM', 'GN');
        $row->set('IDDESTINO', 'PLANEJAMENTO');
        $row->set('IDMOTIVO', $this->getCustomizeMotivo($row));
        $row->set('MOTIVO', $this->getCustomizeMotivo($row));

        $request = new DTO\Request\Row($row, $request->getRoutePath(), $request->getMethod(), $request->getUserId());
        $procRet = $this->tco01100GeneralLibPropCom->notificarPlan($row, $request);
        
        if($procRet instanceof Error){
            $this->notificarPlanErrorHandler($procRet, $response);
        }
    }

    /**
     * Envia resposta ao operador do planejamento.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function enviaResposta(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $row->set('IDORIGEM', 'GN');
        $row->set('IDDESTINO', 'PLANEJAMENTO');
        $row->set('IDMOTIVO', 'DUVIDARESPONDIDA');
        $row->set('MOTIVO', 'DUVIDARESPONDIDA');

        $request = new DTO\Request\Row($row, $request->getRoutePath(), $request->getMethod(), $request->getUserId());
        $procRet = $this->tco01100GeneralLibPropCom->notificarPlan($row, $request);
        
        if($procRet instanceof Error){
            $this->notificarPlanErrorHandler($procRet, $response);
        }
    }

    /**
     * Retorna motivo para customização de acordo
     * com o IDEDITAL da proposta comercial.
     * 
     * @param object $row
     * 
     * @return string 'EDITAL' caso IDEDITAL=='S'
     *  e 'CUSTOMIZACAO' caso não seja.
     */
    private function getCustomizeMotivo($row){
        $proposta = $this->getPropostaComercialByNrVersao($row->get('NRVERSAOPROPCOM'));
        if($proposta && sizeof($proposta)){
            return $proposta[0]['IDEDITAL']=='S' ? 'EDITAL' : 'CUSTOMIZACAO';
        }
        
        return null;
    }

    /**
     * Trata e define no response o erro retornado da 
     * procedure 'SOL_ALT_SERVPROPCOM'.
     * 
     * @param object $procRet
     * @param object $response
     */
    private function notificarPlanErrorHandler($procRet, $response){
        $procRet = explode(' ', $procRet->getMessage());
        $errorMsg = array_values(array_filter($procRet, function($word){
            return strpos($word, '_MSGSTA_')!==false;
        }))[0];

        $errorMsg = explode("\n", $errorMsg);
        $errorMsg = array_filter($errorMsg, function($word){
            return strpos($word, '_MSGSTA_')!==false;
        })[0];

        $errorMsg = str_replace('_MSGSTA_', '', $errorMsg);
        $errorMsg = str_replace('_MSGEND_', '', $errorMsg);

        $response->setError(new Error($errorMsg, -1));
    }

    /**
     * Retorna dados para abertura de widget de
     * serviços complementares.
     * 
     * @param int|float $nrServicoPropCom
     * 
     * @return array dados de serviços complementares.
     */
    public function openServCompl($nrServicoPropCom){
        try{
            $this->entityManager->beginTransaction();

            $procedure = new StoredProcedure($this->connection, 'SERVCOMPLEMENTAR_SERVPROPCOM');
            
            $procedure->addParam(new Param('P_NRSERVICOPROPCOM', Param::PARAM_INPUT,  $nrServicoPropCom,                   Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_CDOPERADOR',       Param::PARAM_INPUT,  $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRORG',            Param::PARAM_INPUT,  $this->environment->getNrOrgTrab(),  Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_CDFILIALLOGADA',   Param::PARAM_INPUT,  $this->environment->getCdFilial(),   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LISTACARD',        Param::PARAM_OUTPUT, null,                                Param::PARAM_TYPE_STR, 10));
            
            $result = $procedure->execute();
            
            if(isset($result['P_LISTACARD']) && $result['P_LISTACARD']){
                $this->entityManager->commit();
                $result = $this->getServComplByNrLista($nrServicoPropCom, $result['P_LISTACARD']);
            } else {
                $result = [];   
            }
            
            return $result;
            
        } catch (\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }

    /**
     * Retorna serviços complementares de acordo com
     * o numero da lista passado por parâmetro.
     * 
     * @param int|float $nrLista
     * @param int|float $nrServicoPropCom
     * 
     * @return array serviços complementares
     */
    public function getServComplByNrLista($nrServicoPropCom, $nrLista){
        $sql = "SELECT
                S.CDCARDAPIOBASE, S.NMSERVICOPROPOSTA,
                S.NRCARDAPIOTRAB, S.NRLISTAERROS,
                S.VRCUSTO, S.NRCARDAPIOBASE, S.NRSERVICOPROPOSTA
            FROM
                (SELECT
                    C.CDCARDAPIOBASE, C.NRCARDAPIOBASE, S.NRSERVICOPROPOSTA,
                    S.NMSERVICOPROPOSTA,
                    CASE
                            WHEN L.CALCULADO = 'S' THEN L.CHAVE
                            ELSE NULL
                        END
                    AS NRCARDAPIOTRAB,
                    CASE
                            WHEN L.CALCULADO = 'S' THEN NULL
                            ELSE L.CHAVE
                        END
                    AS NRLISTAERROS,
                    CASE
                            WHEN L.CALCULADO = 'S' THEN CUSTO_PERC_CARDTRAB(L.CHAVE)
                            ELSE 0.00
                        END
                    AS VRCUSTO,
                    SC.VRCUSTOORIGINAL, SC.VRLIMITECUSTO
                    FROM
                        (SELECT
                                L.CDLISTA, INSTR(L.CDLISTA,'|',1,2) - 1,
                                SUBSTR(L.CDLISTA,1,INSTR(L.CDLISTA,'|') - 1) AS NRCARDAPIOBASE,
                                SUBSTR(L.CDLISTA,INSTR(L.CDLISTA,'|') + 1,INSTR(L.CDLISTA,'|',1,2) - INSTR(L.CDLISTA,'|') - 1) AS CALCULADO,
                                SUBSTR(L.CDLISTA,INSTR(L.CDLISTA,'|',1,2) + 1,50) AS CHAVE
                            FROM LISTA L
                            WHERE L.NRLISTA =:NRLISTA) L
                        INNER JOIN CARDAPIOBASE C ON C.NRCARDAPIOBASE = L.NRCARDAPIOBASE
                        INNER JOIN SERVICOPROPOSTA S ON S.NRSERVICOPROPOSTA = C.NRSERVICOPROPOSTA
                        CROSS JOIN (
                            SELECT
                                P.VRLIMITECUSTO,
                                CUSTO_PERC_CARDTRAB(D.NRCARDAPIOTRAB) AS VRCUSTOORIGINAL
                            FROM
                                SERVICOPROPCOM S
                                INNER JOIN DETALHESERVPROPCOM D ON D.NRDETALHESERVPROPCOM = S.NRDETALHESERVPROPCOM
                                CROSS JOIN PARAMTCO P
                            WHERE
                                S.NRSERVICOPROPCOM =:NRSERVICOPROPCOM
                        ) SC
                ) S
            WHERE S.VRCUSTO <= S.VRLIMITECUSTO / 100 * S.VRCUSTOORIGINAL
                AND S.NRLISTAERROS IS NULL
                AND S.VRCUSTO > 0";

        $params = array(
            "NRSERVICOPROPCOM"=> $nrServicoPropCom,
            "NRLISTA"=> $nrLista
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    /**
     * Seleciona serviços complementares para 
     * o serviço da proposta atual.
     * 
     * @param array $servData Dados do serviço.
     * @param array $servCompl Dados dos serviços
     *  complementares.
     * 
     * @return string ID de validade da operação.
     */
    public function selectServCompl($servData, $servCompl){
        $isValidCreateServCompl = $this->isValidCreateServCompl($servCompl, $servData);
        $existServCompl = $this->getDependentServCompl($servData);        
        $repeatedServCompl = [];
        
        if($isValidCreateServCompl){
            
            foreach ($servCompl as $key => $includeServ) {
                if($existServCompl && sizeof($existServCompl)){

                    foreach ($existServCompl as $key => $existServ) {
                        if($existServ['NRSERVICOPROPOSTA'] == $includeServ['NRSERVICOPROPOSTA'] &&
                        $existServ['NRTIPOPUBLICO'] == $servData['NRTIPOPUBLICO'] &&
                        $existServ['NRPADRAOQUALIDADE'] == $servData['NRPADRAOQUALIDADE']){
                            $repeatedServCompl[] = $includeServ;
                        }
                    }
                }
            }

            if(!sizeof($repeatedServCompl)){
                foreach ($servCompl as $key => $serv) {
                    $this->createServCompl($servData["NRSERVICOPROPCOM"], $serv["NRCARDAPIOBASE"], $serv["NRCARDAPIOTRAB"]);
                }
                return 'VALID';
            } else {
                return $repeatedServCompl;
            }
            
        } else {
            return 'TCO01100_INVALID_QUANT_SERV_COMPL';
        }   
    }
    
    /**
     * Verifica se os serviços complementares selecionados
     * para inserção podem ser inseridos.
     * 
     * @param array $servCompl serviços complementares selecionados
     * @param array $servData Dados do serviço.
     * 
     * @return bool validade da operação
     */
    private function isValidCreateServCompl($servCompl, $servData){
        $qtServicoParamTco = $this->getQtServicoParamTco($servData['QTSEGSEX']);
        
        if($qtServicoParamTco && sizeof($qtServicoParamTco) && !empty($qtServicoParamTco['QTSERVICO'])) 
            $qtServicoParamTco = $qtServicoParamTco['QTSERVICO'];
        else
            $qtServicoParamTco = 0;
        
        $qtServico = sizeof($this->getDependentServCompl($servData));
        
        return (($qtServico+sizeof($servCompl)) <= $qtServicoParamTco);
    }

    /**
     * Retorna quantidade maxima de serviços
     * parametrizada na tabela paramtco.
     * 
     * @param int|float $qtsegsex
     * 
     * @return int|float quantidade de serviços.
     */
    private function getQtServicoParamTco($qtsegsex){
        $sql = 'SELECT MAX(S.QTSERVICO) AS QTSERVICO FROM PARAMTCO P 
            INNER JOIN SERCOMPREFEICAO S ON S.NRPARAMTCO = P.NRPARAMTCO
            WHERE :QTCOMENSAIS BETWEEN QTINICIAL AND QTFINAL';

        $params = array(
            "QTCOMENSAIS" => $qtsegsex
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        return sizeof($result) ? $result[0] : false;
    }

    /**
     * Retorna serviços complementares que 
     * referenciam o serviço da proposta atual.
     * 
     * @param array $servData
     *
     * @return array serviços complementares.
     */
    private function getDependentServCompl($servData){
        $sql = "SELECT * FROM SERVICOPROPCOM 
            WHERE NRSERVICOPROPCOMCOMPL = :NRSERVICOPROPCOM AND NRVERSAOPROPCOM = :NRVERSAOPROPCOM";

        $params = array(
            "NRSERVICOPROPCOM" => $servData['NRSERVICOPROPCOM'],
            "NRVERSAOPROPCOM" => $servData['NRVERSAOPROPCOM']
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        return $result;
    }

    /**
     * Cria serviço complementar referenciando
     * o serviço atual.
     */
    private function createServCompl($nrServicoPropCom, $nrCardapioBase, $nrCardapioTrab){
        try{
            $this->entityManager->beginTransaction();

            $procedure = new StoredProcedure($this->connection, 'CRIA_SERVCOMPLEMENTAR_PROPCOM');
            
            $procedure->addParam(new Param('P_NRSERVICOPROPCOM', Param::PARAM_INPUT,  $nrServicoPropCom,                   Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_NRCARDAPIOBASE',   Param::PARAM_INPUT,  $nrCardapioBase,                     Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_NRCARDAPIOTRAB',   Param::PARAM_INPUT,  $nrCardapioTrab,                     Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_CDOPERADOR',       Param::PARAM_INPUT,  $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRORG',            Param::PARAM_INPUT,  $this->environment->getNrOrgTrab(),  Param::PARAM_TYPE_INT));
            
            $procedure->execute();
            $this->entityManager->commit();
            
        } catch (\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    /**
     * Obtém o último histórico de solicitação
     */
    public function getUltimaSolicitacao($nrServicoPropCom){
        $sql = "SELECT MAX(NRHISTNOTIFPROPCOM) AS NRHISTNOTIFPROPCOM FROM HISTNOTIFPROPCOM
            WHERE NRSERVICOPROPCOM = :NRSERVICOPROPCOM";

        $params = array(
            "NRSERVICOPROPCOM" => $nrServicoPropCom
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $chave = $query->fetchAll();
        $ultima = array();
        if(count($chave) > 0) {
            $sql = "SELECT NRHISTNOTIFPROPCOM, IDMOTIVO, DSOBSERVACAO FROM HISTNOTIFPROPCOM
                WHERE NRHISTNOTIFPROPCOM = :NRHISTNOTIFPROPCOM";
    
            $params = array(
                "NRHISTNOTIFPROPCOM" => $chave[0]['NRHISTNOTIFPROPCOM']
            );
            $query = $this->connection->prepare($sql);
            $query->execute($params);
            $ultima = $query->fetchAll();
        }

        return $ultima;
    }
  
}